/*
 *This file is part of the Configuration File Library (CFL).
 * Copyright (C) 2002 Nicolas Darnis <ndarnis@free.fr>
 *
 * The CFL library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The CFL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the CFL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: cfl.c
 * $Revision: 1.3 $
 * $Date: 2002/10/12 09:24:23 $
 * $Author: ndarnis $
 * $Id: cfl.c,v 1.3 2002/10/12 09:24:23 ndarnis Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <config.h>

#include <gdsl.h>

#include "cfl.h"
#include "parser.h"
#include "section.h"

struct _cfl_handle
{
  gdsl_list_t sections;
};

/******************************************************************************/
/* Public functions                                                           */
/******************************************************************************/

extern cflh_t
cfl_alloc (void)
{
  cflh_t handle;

  handle = (cflh_t) malloc (sizeof (struct _cfl_handle));

  if (handle == NULL)
    {
      return NULL;
    }

  handle->sections = gdsl_list_create ("SECTIONS", section_alloc, section_free);

  if (handle->sections == NULL)
    {
      free (handle);
      return NULL;
    }

  return handle;
}

extern void
cfl_free (cflh_t handle)
{
  gdsl_list_destroy (handle->sections);
  free (handle);
}

extern cflh_t
cfl_read (const char *input_file_name)
{
  cflh_t handle;
  FILE *input_file;

  handle = cfl_alloc ();

  if (handle == NULL)
    {
      return NULL;
    }
  
  input_file = fopen (input_file_name, "r");

  if (input_file == NULL)
    {
      cfl_free (handle);
      return NULL;
    }

  if (yy_parse_file (input_file, handle) < 0)
    {
      fclose (input_file);
      cfl_free (handle);
      return NULL;
    }

  fclose (input_file);

  return handle;
}

extern int
cfl_write (const cflh_t handle, FILE *output_file)
{
  gdsl_list_write (handle->sections, section_write, output_file, NULL);

  return 0;
}

extern int
cfl_write_xml(const cflh_t handle, FILE *output_file)
{
  fprintf (output_file, "<CFL REF=\"%p\">\n", handle);
  gdsl_list_map_forward (handle->sections, section_display, (void *) output_file);
  fprintf (output_file, "</CFL>\n");

  return 0;
}

extern int
cfl_section_insert (cflh_t handle, const char *section_name)
{
  if (gdsl_list_search_by_value (handle->sections, section_compare, 
				 (void *) section_name) != NULL)
    {
      return -1;
    }

  if (gdsl_list_insert_tail (handle->sections, (void *) section_name) == NULL)
    {
      return -2;
    }

  return 0;
}

extern int
cfl_section_remove (cflh_t handle, const char *section_name)
{
  cfl_section_t s;

  s = (cfl_section_t) gdsl_list_remove_by_value (handle->sections, 
						 section_compare, 
						 (void *) section_name);
  if(s == NULL)
    {
      return -1;
    }

  section_free (s);

  return 0;
}

extern int
cfl_get_sections_count (cflh_t handle)
{
  return gdsl_list_get_card (handle->sections);
}

extern char *
cfl_section_get_name_by_position (cflh_t handle, unsigned long pos)
{
  cfl_section_t s;

  s = gdsl_list_search_by_position (handle->sections, pos);
  if (s == NULL)
    {
      return NULL;
    }

  return section_get_name (s);
}

extern long
cfl_section_get_values_count (const cflh_t handle, const char *section_name) 
{
  cfl_section_t s;

  s = (cfl_section_t) gdsl_list_search_by_value (handle->sections, 
						 section_compare, 
						 (void *) section_name);

  if (s == NULL)
    {
      return -1;
    }

  return (long) section_get_values_number (s);
}

extern int
cfl_value_insert (cflh_t handle, const char *section_name, 
		  const char *value_name, const cfl_value_type_t type, void *val)
{
  int rc;
  cfl_section_t s;

  s = (cfl_section_t) gdsl_list_search_by_value (handle->sections, 
						 section_compare, 
						 (void *) section_name);

  if (s == NULL)
    {
      return -1;
    }

  rc = section_add_value (s, value_name, type, val);

  if (rc < 0)
    {
      return rc;
    }
  
  return 0;
}

extern int
cfl_value_remove (cflh_t handle, const char *section_name, 
		  const char *value_name)
{
  int rc;
  gdsl_element_t e;

  e = gdsl_list_search_by_value (handle->sections, 
				 section_compare, 
				 (void *) section_name);

  if (e == NULL)
    {
      return -1;
    }

  rc = section_remove_value ((cfl_section_t) e, value_name);

  if (rc < 0) 
    {
      return rc;
    }

  return 0;
}

extern int
cfl_value_set_by_name (cflh_t handle, const char *section_name, 
		       const char *value_name, const cfl_value_type_t type, void *val)
{
  int rc;
  cfl_section_t s;

  s = (cfl_section_t) gdsl_list_search_by_value (handle->sections, 
						 section_compare, 
						 (void *) section_name);

  if (s == NULL)
    {
      return -1;
    }

  rc = section_set_value (s, value_name, type, val);

  if (rc < 0)
    {
      return rc;
    }
  
  return 0;
}

extern void *
cfl_value_get_by_name (cflh_t handle, const char *section_name, 
		       const char *value_name, cfl_value_type_t *type)
{
  cfl_section_t s;

  s = (cfl_section_t) gdsl_list_search_by_value (handle->sections, 
						 section_compare,
						 (void *) section_name);

  if (s == NULL)
    {
      return NULL;
    }

  return section_get_value (s, value_name, type);
}

extern void *
cfl_value_get_by_position (const cflh_t handle, const char *section_name, 
			   unsigned long pos, char **value_name, 
			   cfl_value_type_t *type)
{
  cfl_section_t s;

  s = (cfl_section_t) gdsl_list_search_by_value (handle->sections, 
						 section_compare, 
						 (void *) section_name);

  if (s == NULL)
    {
      return NULL;
    }

  return section_get_value_by_pos (s, pos, value_name, type);
}

